function w = beamwidth (Dx,Dy,Mx,My,phi0,th0,level)

%W = BEAMWIDTH (DX,DY,MX,MY,PHI0,TH0,LEVEL)
%
% Find the (up-down) beamwidth, in the vertical phi=PHI0 plane, 
% of a 2D-phased array with element spacing DX and DY (in units of
% wavelength) and MX and MY element in the X- and Y-directions,
% phase steered to direction PHI0 and TH0 (radians).
% TH0 can be a vector.
% 
% The returned beamwidth W corresponds to the width of
% the beam at the LEVEL, which must be  0 < LEVEL < 1.
% If LEVEL is not given, 0.5 is used.
%
% We solve for th+, th- the beamshape equation 
%
%   diric(psiX,Mx) * diric(psiY,My) = +- sqrt(level)  
% 
% with
%
%   psiX = 2*pi*Dx*cos(psi0) * (u-u0)
%   psiY = 2*pi*Dy*sin(psi0) * (u-u0)
%
% where u = sin(th), u0 = sin(th0).
%
% Values of TH0 that are too near to +-pi/2 return W=NaN.

%
% 21-Oct-2005 Jm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 6 || nargin > 8
    error ('Bad number of arguments')
elseif nargin == 6
    level = 0.5;
end

% First find the value X (= u - u0) where
%
%   diric(A*X,Mx)*diric(B*X,My) = sqrt(level)

A = abs(2*pi*cos(phi0)*Dx);
B = abs(2*pi*sin(phi0)*Dy);

if A ~= 0
    a = 1/(Mx*A);
else
    a = Inf;
end
if B ~= 0
    b = 1/(My*B);
else
    b = Inf;
end
X0 = [0, 2*pi*min([a,b])];

X = fzero (@(x) fun(x,A,B,Mx,My,sqrt(level)), X0);

% Then find the points u1 and u2 where
%
% {+/-}X = u{2/1} - u0 

u0 = sin (th0);
u1 = u0 - X;
u2 = u0 + X;

% Then convert from u to theta.

th2 = asin(u2);
th1 = asin(u1);

w = th2 - th1;

% Flag the illegal points as NaNs.

K1 = find (abs(u1) > 1);
K2 = find (abs(u2) > 1);
K = union(K1,K2);
w(K) = NaN;

end %beamwidth


function y = fun (x,A,B,Mx,My,level)
%-------------------------------------
    y = abs (diric (A*x,Mx) * diric (B*x,My)) - level;
end

