function Vol = common_volume

% COMMON_VOLUME
%
% Common volume size for the VHF - Kiruna Test Array system.

% 21-Sept-2006 Jm
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

set(0,'defaultaxesfontsize',12);

H = 120;    


d2r = pi/180;
r2d = 180/pi;

%E_PHI = 0*d2r;         % Element azimuth (CCW from North) (radians)
%E_TH  = 35*d2r;        % Element's zenith angle (radians)
                        % Value not really used: we compute the test array
                        % relative gain using only the array factor, that is,
                        % we assume the element beam is wide enough.   

PHI0  = 0;              % Test array beam azimuth (CCW from North)

if H == 120

    % Test array beam zenith angle (radians)

    TH0   = 60.1*d2r;         

    % Grid boundaris, km from the beam intersection point
    
    Nlim = [-15,15];        % North-South
    Wlim = [-5,5];          % East-West
    Zlim = H + [-40, 40];   % Up-Down

    % For speed, scale basic grid steps (defined below) by

    scaler = 2;

elseif H == 300.6

    % Test array beam zenith angle (radians)

    TH0   = 35*d2r;         

    % Grid boundaris, km from the beam intersection point
    
    Nlim = [-15,15];        % North-South
    Wlim = [-5,5];          % East-West
    Zlim = H + [-45, 55];   % Up-Down

    % For speed, scale basic grid steps (defined below) by

    scaler = 2;

elseif H == 600

    TH0   = 20*d2r;         % Test array beam zenith angle (radians)

    Nlim = 2*[-15,15];
    Wlim = 2*[-5,5];
    Zlim = H + 2*[-60, 70];

    % For speed, scale grid steps by

    scaler = 5;

else
    error (['Unsupported height value (' num2str(H) ')'])
end

L = H*tan(TH0)        % Kiruna-Tromso distance in flat-Earth model.

Range = H/cos(TH0)    % Range from the test array to the common volume.

% -------------------------------------------------------------------------

RV0 = H;                % Range from T to the center of the common volume
RK0 = Range;            % Range from K to the center of the common volume

% Common volume-covering grid over T
%   X is towards North,
%   Y is towards West,
%   Z is upwards

% Basic grid spacing in km (actual spacing affected by SCALER)

dX = 0.3;
dY = 0.1;
dZ = 0.5; 

% The grid points in Tromso coordinate system

X = Nlim(1):(scaler*dX):Nlim(2);
Y = Wlim(1):(scaler*dY):Wlim(2);
Z = Zlim(1):(scaler*dZ):Zlim(2);

% Size of a volume element

dV = (X(2)-X(1))*(Y(2)-Y(1))*(Z(2)-Z(1))
dA = (X(2)-X(1))*(Y(2)-Y(1))

% Find index [nx0,ny0,nz0] of the grid point that is as near as possible to the
% common volume center 

[dummy,nx0] = min (abs( X - 0));
[dummy,ny0] = min (abs( Y - 0));
[dummy,nz0] = min (abs( Z - H));

% Number of grid points

Nx = length(X);
Ny = length(Y);
Nz = length(Z);
N = Nx * Ny * Nz

% We assume flat Earth.
% Then the cartesian coordinates [XX,YY,ZZ] of the common volume grid
% points in the test-array's coordinate system are

XX = X + L;
YY = Y;
ZZ = Z;

% Test array is a 12 x 4 array, with uniform 1.4 wavelength spacing.

AD = create_arraydesc (12, 4, 1.4, 1.4);   % Mx, My, Dx, Dy

% Allocate storage arrays:
%  GG holds the gain product
%  PPD holds propagation delay (total distance) of the grid point
%  Az is effective intersection area (sum of GG at fixed Z)

GG = zeros(Nx,Ny,Nz);
PPD = zeros(size(GG));
Az = zeros(1,Nz);


getfigure ('Z-slice', [1 1 6 6],400);
clf
subplot(2,1,2)  % This subplot is the Z-slice profile.
subplot(2,1,1)  % This subplot is the Z-slice contours. 

% Compute all the terms GG(P) in the volume sum to memory

for nz = 1:length(Z)
    for ny = 1:length(Y)
        for nx = 1:length(X)

            % Grid point in Tromso coordinate system
            %   X(nx) is North from Tromso 
            %   Y(ny) is West from Tromso 
            %   Z(nz) is Up from Tromso

            r = [X(nx),Y(ny),Z(nz)];

            % Distance from Tromso

            RV = norm(r);

            % VHF relative gain at the grid point

            Gain_VHF = simple_vhfbeam ( Y(ny)/RV, X(nx)/RV ); 

            % Grid point's spherical coordinates in Kiruna system

            [PHI,EL,RK] = cart2sph ( XX(nx) ,YY(ny), ZZ(nz) ); 

            % Test array's relative gain at the grid point 

            AF_KIR = arraygain ( PHI, pi/2 - EL, PHI0, TH0, AD, []);
            Gain_KIR = AF_KIR^2;

            % Save the gain product and propagation delay of the grid point

            GG(nx,ny,nz) = Gain_VHF * Gain_KIR / ( (RV/RV0)^2 * (RK/RK0)^2 );
            PPD(nx,ny,nz) = RK+RV; 
        end
    end

    % Effective area and contour curves of current z-slice

    Az(nz) = sum(sum(GG(:,:,nz)))*dA;
    plot_gain (GG(:,:,nz), X, Y, ZZ(nz),  Az(nz), [0.9 0.5 0.3 0.2 0.1 0.01],'contour');
end 

% Replot the contours of the central Z-slice

subplot (2,1,1)
    plot_gain (GG(:,:,nz0), X, Y, ZZ(nz0), Az(nz0), [0.9 0.5 0.1 0.01], 'contour');

% Plot the effective area profile A(z)

subplot (2,1,2)
    h = plot (Az,Z); set(h,'linewidth',1.5);
    xlabel ('Effective area (km^2)')
    ylabel ('Altitude (km)')
    set_axopt

% Compute the common volue size

Vol = sum(GG(:)) * dV

% Find out the filled scattering volume as function of pulse propagation
%
%   We just find out for any given PPD which points are scattering up to
%   that time, and add the gain products of those points. The quantity F(PPD)
%   is the integrated volume. 

ppd = PPD(:);
gg = GG(:);

us2km = 0.3;
km2us = 1/us2km;

PPD1 = min(ppd);
PPD2 = max(ppd);
PPD0 = PPD(nx0,ny0,nz0);

% Assume a Scale x 10 usec pulse

Scale = 7;             % Pulselen in units of 10 us (must be an integer)
Pulselen = Scale*10;   % Pulselen in microsec
dPPD = Pulselen*us2km; % Pulselen in km

% Form a PPD grid such that the actual PPD of the beam intersection point
% sits in the middle of some interval [PPD(k) PPD(k+1)], so that we will
% find the true maximal volume.

dd = max ( [PPD2-PPD0, PPD0-PPD1] );
d1 = PPD0-(dPPD/2)-floor(dd/dPPD) * dPPD
d2 = PPD0+(dPPD/2)+floor(dd/dPPD) * dPPD; 

delay = d1:dPPD:d2; 

% Find the accumulated volume F = F(PPD)

F = zeros(1,length(delay)); 
for k = 1:length(delay)
    K = find (ppd < delay(k));
    F(k) = sum (gg(K))*dV;
end

% GRAPH the volume's filling
% ===================================================

getfigure ('Scattering volume', [1 1 6 6],400);
subplot(2,1,1)
    h = plot(delay*km2us, F,'-'); set(h,'linewidth',1.5);

    x = ( min(delay) + (max(delay)-min(delay))*0.7 ) * km2us;
    y = max(F)*0.5;
    text(x,y,sprintf('V = %1.0f km^3',Vol));

    xlabel('PPD (\mus)')
    ylabel('V_{eff}  for a long pulse (km^3)')
    set_axopt

subplot(2,1,2)
    delay(end) = [];
    dF = diff(F);
    [mx,mk] = max(dF);
    h = plot(delay*km2us, dF,'o'); set(h,'linewidth',1.5);

    x = delay(mk)*km2us;
    y = mx*0.3;
    h = text(x,y,sprintf('V = %1.0f km^3',max(dF)));
    set (h, 'hor','cent');

    xlabel('PPD (\mus)')
    ylabel('V_{eff}  for a 70 \mus pulse (km^3)')
    set_axopt



%=============================================================
%   SUBROUTINES
%=============================================================


function plot_gain (Gz, X,Y, z, A, Levels, mode )
%--------------------------------------------------------
%
%--------------------------------------------------------
if max (Gz) < min(Levels)
    [cc,hh] = contour (X, Y, Gz);% clabel(cc);
else
    [cc,hh] = contour (X, Y, Gz, Levels); clabel(cc);
end

set(hh,'linewidth',2);    
axis equal
title (sprintf('Gain product at z = %1.1f km  (Aeff = %1.1f)',z, A));
xlabel('South - North (km)')
ylabel('East - West (km)')
set_axopt
drawnow


function set_axopt
%--------------------------------------------------------
%
%--------------------------------------------------------
%set (gca,'pos',[0.17 0.15 0.7750 0.5]);
set(gca,'tickdir','out');
set(gca,'layer','top');
set(gca,'tickdir','out');
set(gca,'ticklen',[0.02,1]);

