function linearray

% Inspect the far field of a uniform array of "yagis".
%
% The Nx x My array is in the x-y plane. We compute the array
% pattern in the xz-plane. When the beam is phase-steered, we allow
% tilting only in the xz plane, so that we can treat the array 
% essentially as one-dimensional.
%
% 
%

% 13-Oct-2005 Jm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
set(0,'defaultaxesfontsize',14);
d2r = pi/180;
r2d = 180/pi;


% ==================================================================

Mx = 10;              % Number of elements in x direction
My =  1;              % Number of elements in y direction  

D = 1.50;             % Element distance devided by wavelenth
delta = -pi;          % Phasing step between the elements, radians
                      % Must be between -pi and pi.  

Ybw = 40*d2r;         % Full beamwidth of an individual "yagi", radians.
                      % We assume power gain Gy of the form
                      % Gy = Gy0*[cos(theta/2)]^ny. The factors ny and Gy0
                      % are computed below, to give the required BW.
                      % (The beamwidth refers to the 0.5*G0 beamwidth, not
                      % to the 3dB beamwidth.) 

theta1 = -pi;         % Zenith-angle grid
theta2 =  pi;
Ntheta = 1000;

% ==================================================================


theta = linspace (theta1,theta2,Ntheta);


% Max possible phase-steared offset for the given D occures with delta = -180,
% and is max_offset = arcsin ( 1/2D ), when D >= 1/2.   

if D > 0.5
    max_offset = asin ( 1/(2*D) ) * deg;
else
    max_offset = 90;
end

% Compute the element's directivity (=max power gain) G0, assuming 
% insignificant side-lobes in the cos^n (theta/2) beam power gain pattern.
% (This is found by requiring that the pattern integrates to 4pi.)

n = log10 (0.5) / log10 (cos(element_wd*rad/2));
G0 = (n+2)/2;

% Element's power directivity in dB

G0_dB = 10*log10(G0);

% Compute the "effective diameter" deff of the element. We define this
% to be the diameter of a circle which has the area equal to the
% effective area Aeff of the element, Aeff = G lambda^2/4pi. Then we
% define Deff = deff / lambda.

Deff = sqrt(G0)/pi;

% Element's normalized _amplitude_ gain for all elevations

theta = pi/2 - elevation;

G1 = cos ( theta/2 ) .^ (n/2);

% Directions of the grating beams
%
%   2 pi D cos (epsilon) + delta = n 2 pi

N = [-2 -1 1 2];
grating_dir =  [];
for n = N
    cos_elevation = (n*2*pi - delta*rad) / (D*2*pi);
    if abs(cos_elevation) <= 1
        grating_dir = [grating_dir acos(cos_elevation)];
    end
end
grating_dir*deg;

% Direction of the main beam
%
%   2pi*D*cos(epsilon) + delta = 0

N = 0;
cos_elevation = (N*2*pi - delta*rad) / (D*2*pi);
if abs(cos_elevation) <= 1
    main_dir = acos(cos_elevation);
else
    main_dir = [];
end

% Width of the main beam, at the given phase-stearing delta, from null to null
%
%   cos elev(+-) = -delta +- 2pi/M
%
%   BW00 = abs(elev(+) - elev(-))

x1 = (delta*rad + 2*pi/M) / (2*pi*D );
x2 = (delta*rad - 2*pi/M) / (2*pi*D );    
BW00 = abs (acos(x1) - acos(x2)) * deg;

% Array factor AF

Delta = D*cos(elevation)*2*pi + delta*rad;
AF = abs(diric(Delta,M));


%=======================================================================
%               GRAPHS
%=======================================================================

getfig('linearray')
set (gcf, 'paperpos', [0.25 2.5 7 8]);

% Figure header

h1 = subplot('position',[0.13,0.85,0.7750,0.10]);
hdr{1} = sprintf('Array M=%d  d/\\lambda=%1.2f  Stearing \\delta=%1.1f\\circ', ...
                                                            M, D, -delta);

fmt1 = 'Element BW=%1.1f\\circ';
fmt2 = '  Directivity=%1.1fdBi';
fmt3 = '  Deff/\\lambda=%1.2f';
hdr{2} = sprintf ([fmt1 fmt2 fmt3], 2*element_wd, G0_dB, Deff);

m_str = sprintf ('%1.1f\\circ ',main_dir*deg);
g_str = sprintf ('%1.1f\\circ ',sort(grating_dir)*deg);
w_str = sprintf ('%1.2f\\circ ',BW00);
hdr{3} = ['Beam \epsilon=' m_str ' BW00=' w_str '  Grating ' g_str];

header(h1, hdr);

% Array factor and single-element gain factor for all elevation.
% We also plot a +- 0.6 deg stripe at elevation 90 to serve as direction
% reference. (+- 0.6 deg is eiscat uhf beam between first minima).

h2 = subplot('position',[0.13,0.5,0.7750,0.3]);
  plot (elevation*deg,AF,'b', elevation*deg, G1, 'r--');
    hold on  
  h_area = area ( [89.4 90.6], [1.05 1.05]);  
    set(h_area,'facecolor',[1 1 1]*0.8);
    set(h_area,'edgecolor',[1 1 1]*0.8);
  plot (elevation*deg,AF,'b', elevation*deg, G1, 'r--');
    hold off;
    set(gca,'layer','top');
    set(gca,'tickdir','out');
    set(gca,'xlim',[elev1,elev2]);
    set(gca,'ylim',[0,1.05]);
    xlabel('Elevation (\circ)');
    ylabel('G_1 & AF');

% Total antenna pattern

h3 = subplot('position',[0.13,0.14,0.7750,0.25]);
  plot (elevation*deg,G1.*AF,'b');
    set(gca,'layer','top');
    set(gca,'tickdir','out');
    set(gca,'xlim',[elev1,elev2]);
    set(gca,'ylim',[0,1.05]);
    xlabel('Elevation (\circ)');
    ylabel('G_1 \times AF');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%               SUBFUNCTIONS
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function header (h,txt)
%============================================================
% 
%============================================================
cla
Eventnum = [];
set(h,  'xticklabel',[],...
        'yticklabel',[],...
        'xtick',[],...
        'ytick',[],...
        'box','on',...
        'color',[1 1 1]);

        
N = length(txt);        
n = N:-1:1;
y = (2*n-1)/(2*N);
for k = 1:N
    hh = text(0.02,y(k),txt{k});
    set(hh,...
        'Color',[0 0 0],...
        'VerticalAlignment','middle'...
        );
        set(hh,'fontsize',14);

end    

if ~isempty(Eventnum)

    pos = get(h,'pos'); 

    wd = 0.07;
    hgt = 0.03;
    x1 = pos(1)+pos(3) - wd;
    y1 = pos(2)+pos(4) - hgt;

    h = axes('position',[x1,y1,wd,hgt]);
    set(h,  'xticklabel',[],...
            'yticklabel',[],...
            'xtick',[],...
            'ytick',[],...
            'box','on',...
            'color',[1 1 1]);
    h = text(0.5,0.5,num2str(Eventnum));
    set(h,'horizontalAlignment','center');
    set(h,'verticalAlignment','middle');
end

%header


function hh = getfig(Figurename)
%----------------------------------------
%
%----------------------------------------
Colordef = 'white';
Width1 = 500;
Height1 = 300;

if nargin == 0
    nfig = length(get(0,'Children'));
    Figurename = ['fig.' num2str(nfig+1)];
    Size = 1;
elseif nargin == 1
    Size = 1;
else
    error('wrong number of input arguments')
end 

if ~isnumeric(Size)
    Size = str2num(Size);
end    
Width = ceil(Size*Width1);
Height = ceil(Size*Height1);

h = findobj('Tag',Figurename);
if isempty(h)
    h = figure;
    set(h,'Tag',Figurename);
    set(h,'Numbertitle','off');
    colordef(h,Colordef);
    pos = get(h,'Position');
    set(h,'Position',[pos(1), pos(2), Width, Height]);
    set(h,'PaperType','A4');
    set(h,'Name',Figurename);
    
else
    figure(h);
end

if nargout > 0
    hh = h;
end    

