pfunction planearray (PHI0, THETA0)

%PLANEARRAY (PHI0,THETA0)
% Inspect the far field of a uniform xy-array of tilted elements.
%
% The direction of the phase-steered beam PHI0, THETA0 (radians)
% can optionally be given in the command line, all other parameters
% must be provided by edited in the input section at the beginning
% of the source file PLANEARRAY.M.  
%
% See also plot_planearray.m

%  5-Nov-2005 Jm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

set(0,'defaultaxesfontsize',14);
d2r = pi/180;
r2d = 180/pi;

deltaX = [];
deltaY = [];
theta0 = [];
phi0   = [];
Ewidth = [];
Egain  = [];
PLOT_3D = 0;


%===================== BEGIN INPUT =====================================

Mx = 50;              % Number of elements in x direction.
My = 20;              % Number of elements in y direction.  

Dx = 1.50;            % Element distance in x-direction devided by wavelenth.
Dy = 1.50;            % Element distance in y-direction devided by wavelenth.

%             Specify either deltaX,Y OR theta0,phi0  (not both)
%             ---------------------------------------
                    
%deltaX = 280*d2r;    % Phasing step between the elements in x direction (rad).
%deltaY = 000*d2r;    % Phasing step between the elements in y direction (rad).  

phi0   = 00.0*d2r;        % Desired beam direction, "azimuth" angle (radians)
theta0 = 00.0*d2r;        % Desired beam direction, polar angle (radians)

%             Specify either the element beamwidth or the power gain (not both)   
%             -----------------------------------------------------
                        
%Ewidth = 40*d2r;       % Beam width of an individual element, radians.
                        % We assume power gain Gy of the form
                        % Gy = Gy0*[cos(theta/2)]^ny. The factors ny and Gy0
                        % are computed below, to give the required BW.
                        % (The beamwidth refers to the 0.5*Gy0 beamwidth, not
                        % to the 3dB beamwidth.) 

Egain = 10.00;          % Power gain as a multiplicative power factor, _not_ in dB !
                        % For instance, use 23.44 (= 10^1.37) instead of 13.7 (dB). 

Ephi = 00*d2r;        % Azimuth and polar angle (zenith angle) of the yagi,
Eth  = 00*d2r;        % in radians. The yagi should not be tilted more than   
                      % about Ybw from vertical, otherwise the beam
                      % starts seriously to touch the ground, and this
                      % case is not accounted for here in any way.
                      % The "azimuth"  is as defined in Matlab, counter-
                      % clockwise from positive x-axes (towards positive y). 

PhiV = [  ]*d2r;      % Azimuth angle (radians) of the vertical plane where
                      % the beam pattern is plotted. If PhiV == [], we 
                      % use PhiV = phi0, the azimuth of the selected beam. 

PLOT_3D = 0;          % Flag. If set, make 3D plot of the grating panel
                      % to a separate window. 

%===================== END INPUT =======================================

% If nargin == 2, the given PHI0 and THETA0 override the values given
% above in this file.
if nargin == 2
    deltaX = [];
    deltaY = [];
    phi0 = PHI0;
    theta0 = THETA0;
end

% Check the array element description

ED.phi   = Ephi;
ED.th    = Eth;
ED.width = Ewidth;
ED.gain  = Egain;
ED = check_array_element (ED);
Yth      = ED.th;
Yphi     = ED.phi;
Ybw      = ED.width;

% If deltaX and deltaY are non-empty, use them directly, else try to
% compute them from the given required beam direction.

if isempty(deltaX) || isempty(deltaY)
    if isempty(phi0) || isempty(theta0)
        error ('Either deltaX,deltaY or theta0,phi0 must be specified')
    end
    deltaX = 2*pi * sin(theta0)*cos(phi0) * Dx;
    deltaY = 2*pi * sin(theta0)*sin(phi0) * Dy;
end

dBmin  =   -30;       % yxis lower limit for G/Gmax plot.

% Polar-angle grid

theta1 = -pi/2;
theta2 =  pi/2;
theta_step = 0.02*d2r; 
theta = theta1:theta_step:theta2;

thlim = [-4 4]*d2r;  % theta ax limits for "Main beam" plot   
                     % If thlim is empty, use defaults
                     % that are proportional to the actual beam width.

% Compute the element's directivity (=max power gain) G0, assuming 
% insignificant side-lobes in the cos^ny (theta/2) beam power gain pattern.
%
% THIS IS NOW DONE IN CHECK_ARRAY_GAIN.M, so we  just pick the values here.

ny = ED.n;
Ge0 = ED.gain;

% Element's directivity in dB

Ge0_dB = 10 * log10 (Ge0);

% Main beam direction and grating lobe directions in the array factor

[Ux,Uy,Uz] = grating_dir (Dx,Dy, deltaX, deltaY);
[Phi, El] = cart2sph(Ux,Uy,Uz);
Th = pi/2 - El;

Phi0 = Phi(1);
The0 = Th(1);

% Define the azimuth "phi" of the the vertical plane where the beam patterns
% are to be plotted

if isempty (PhiV)
    phi = Phi0;
else
    phi = PhiV;
end

% Unit circle to be plotted in the "grating" panel.

[Cx,Cy] = pol2cart (linspace (0,2*pi,100),1);

% Line to be plotted in the grating panel to indicate the V-plane direction.

[CCx,CCy] = pol2cart ([phi phi+pi],1);

% Gain Ge of the tilted element in direction r.
%
% The gain is
%
%   Ge = Ge0 * cos (th1/2) .^ ny;
%
% where th1 is the angle between yagi direction vector Ue and the
% direction vector r.
%
% The cosine of the half angle th1/2 can be expressed via the dot product
% of ry and r, 
%
%   cos (th1/2) = +sqrt { (1 + r*Ue')/2 }.
%
% We compute the element gain in the vertical plane -- fixed phi, Yphi and Yth --
% as a function of the zenith angle theta.

Ue = [sin(Yth)*cos(Yphi), sin(Yth)*sin(Yphi), cos(Yth)];

% For plotting into the "Grating" panel, compute the XY-projection of 
% the intersection of the element's half-power cone with the unit sphere.

Ecirc = conecircle ( Ue, Ybw/2, 100);
EcircX = Ecirc(1,:);
EcircY = Ecirc(2,:);

% Matrix rr, where a column gives the direction vector r for a single theta.

rr = [sin(theta)*cos(phi); sin(theta)*sin(phi); cos(theta)];

% An element's normalized gain as function of theta, as a row vector

Ge = (0.5 * (1 + Ue*rr)) .^ (ny/2);
Ge_dB = 10*log10(Ge);

% Array factor, assuming all a_m = 1.

Mxy = Mx*My;

PsiX = 2*pi*Dx*cos(phi)*sin(theta) - deltaX;
PsiY = 2*pi*Dy*sin(theta)*sin(phi) - deltaY;
AFx = abs(diric(PsiX,Mx));
AFy = abs(diric(PsiY,My));
AF = AFx .* AFy .* Mxy;

Ga_dB = 20*log10(AF) - 20*log10(Mxy);

% Compute our "reference gain"

Gref = Mxy * ED.gain;
Gref_dB = 10*log10(Gref);

% Compute the power integral PI for this phase steering:

%  - Find the elements' gain in the grating directions g (using
%    the to unity normalized relative gain of the element).

for g = 1:length(Phi)
    GE(g) = simplegain (Phi(g), Th(g), ED);
end

% - Compute the grating sum

Grsum = sum ( GE ./ cos(Th) );

% - The power integral is

PI = Grsum * Mxy / ( 4*pi*Dx*Dy );

% Compute the grating lobe gains for this phase-steering

Gr = GE*Mxy^2 / PI;
Gr_dB = 10*log10(Gr) - Gref_dB;    % in dB, compared to the reference gain.

% Array's directivity DI for this phase-steering is the max of the Gr.
% (Note that our "marker" lobe might not be the lobe which
%  corresponds to DI. We will nevertheless show the marker lobe info,
%  but in addition, we will show the strongest beam if marker beam is
%  not the strongest beam, and the next strongest beam, if marker beam
%  is the strontest beam.) 

[Gr_sorted,K] = sort(Gr,2,'descend');
if K(1) == 1
    % Beam 1, the marker beam, is the strongest beam
    DI = Gr_sorted(1:2);
    KI = K(1:2);
else
    % Beam 1, the marker beam, is not the strongest beam
    DI = [Gr(1), Gr_sorted(1)];
    KI = [1, K(1)];         
end
DI_dB = 10 * log10 (DI);

% The absolutely normalized antenna gain is G = Ge * AF^2 / PI,
% And here we normalize it by dividing by Gref.

G = Ge .* AF.^2 / (PI*Gref);
G_dB = 10*log10(G);

% Irrespective of what V-plane we are plotting, we always compute the
% beam width, BW0, in the vertical plane that goes through the beam center.

BW0 = beamwidth (Dx,Dy, Mx, My, Phi0, The0, 0.5);

% When the V-plane does not go through the main beam, we just pick for
% plotting a segment of the theta axis around the largest gain value in
% the plane.

[Gmax,k] = max(G);

[Gmax, k, k1, k2] = beamparam (G, k, 0.5);
Gmax_dB = 10*log10(Gmax);

BW = abs (theta(k2)-theta(k1));
Bdir = theta(k);
if isempty(thlim)   
    theta_main1 = -3*BW;
    theta_main2 = 3*BW;
else
    theta_main1 = thlim(1);
    theta_main2 = thlim(2);
end

% Aeff / A, for the strongest beam (which might not be the marker beam).

AeffPerA = max(DI)/(4*pi*Mx*Dx*My*Dy);

% Generate some info strings about the grating lobes, to be printed in 
% the figure header. Do some optimization in string placement depending
% on how many grating lobes we have.

gs_12 = '';
for m=1:min([2,length(Phi)])
    if m == 1
        fmt1 = '[%d] \\phi=%1.0f\\circ \\theta=%1.0f\\circ';
            fmt2 = '  G/Gref=%1.1f dB   ';
        u  = sprintf ( [fmt1 fmt2], ...
                            m, Phi(m)*r2d, Th(m)*r2d, ...
                                Gr_dB(m));
    else
        u  = sprintf ('[%d] %1.0f %1.0f %1.1f   ',...
                            m, Phi(m)*r2d, Th(m)*r2d, Gr_dB(m));
    end
    gs_12 = [ gs_12 u ];
end
gs_35 = '';
for m=3:min([5,length(Phi)])
    u  = sprintf ('[%d] %1.0f %1.0f %1.1f   ',...
                        m, Phi(m)*r2d, Th(m)*r2d, Gr_dB(m));
    gs_35 = [ gs_35 u ];
end
gs_6end = '';
for m=6:length(Phi)
    u  = sprintf ('[%d] %1.0f %1.0f %1.1f   ',...
                        m, Phi(m)*r2d, Th(m)*r2d, Gr_dB(m));
    gs_6end = [ gs_6end u ];
end


%=======================================================================
%               GRAPHS
%=======================================================================

getfigure('plane-array (1)',[0.25 2.5 7 8], 550);

% Header

h1 = subplot('position',[0.05,0.73,0.8550,0.19]);

    % line 1

    fmt1 = 'ARRAY       MxMy=%d{\\times}%d';
      fmt2 = '  Dx=%1.2f  Dy=%1.2f';
        fmt3 = '  Steering {\\delta}x=%1.1f\\circ  {\\delta}y=%1.1f\\circ';
    hdr{1} = sprintf ([fmt1 fmt2 fmt3], ...
                Mx, My, ...
                  Dx, Dy, ...
                    deltaX*r2d, deltaY*r2d);

    % line 2

    fmt1 = 'ELEMENT   \\phi=%1.1f\\circ \\theta=%1.1f\\circ';  
      fmt2 = '  w=%1.1f\\circ';
        fmt3 = '  G=%1.1f dBi';
          fmt4 = '  Aeff/A=%1.2f';
    hdr{2} = sprintf ([fmt1 fmt2 fmt3 fmt4], ...
               Yphi*r2d, Yth*r2d, ...
                 Ybw*r2d, ...
                   Ge0_dB, ...
                     AeffPerA);

    % line 3 
    
    fmt1 = 'BEAM 1      \\phi=%1.1f\\circ \\theta=%1.1f\\circ';
      fmt2 = '  w=%1.2f\\circ';
        fmt3 = '  D%d=%+1.1f dB  D%d=%+1.1f';
           fmt4 =  '  Gref=%1.1f dBi';
    hdr{3} = sprintf ([fmt1 fmt2 fmt3 fmt4], ...
                Phi0*r2d, The0*r2d, ...
                  BW0*r2d, ... 
                    KI(1), DI_dB(1)-Gref_dB, KI(2), DI_dB(2)-Gref_dB, ...
                       Gref_dB ); 

    % line 4

    hdr{4} = sprintf ('GRATING     %s  ',  gs_12 );

    % line 5

    hdr{5} = sprintf ('                  %s ',  gs_35 );

    % line 6

    hdr{6} = sprintf ('                  %s ', gs_6end);


    figureheader(h1, hdr);

% Element gain, Array factor

subplot('position',[0.15,0.47,0.7550,0.2*7/8]);
    thetadeg = theta*r2d;
    h = plot (thetadeg, Ge_dB, 'r', thetadeg, Ga_dB,'b');
    set (h(1), 'linewidth',2);
    set (h(1), 'color',[1 0.6 0.6]);
    set (gca,'layer','top');
    set (gca,'tickdir','out');
    set (gca,'xtick',-90:30:90);
    set (gca,'ticklength',[0.015 0.025]);
    set (gca,'xlim',[theta1,theta2]*r2d);
    set (gca,'ylim',[dBmin,1.0]);
    xlabel ('\theta (\circ)');
    ylabel ('(dB)') 
    phi_str = sprintf('\\phi=%1.1f\\circ',phi*r2d);
    h = title (['Element gain and array factor in ' phi_str ' plane']); 
    set (h, 'fontweight','b');


% Grating

%%%getfigure('plane-array (3)', [0.25 2.5 3 3], 300);
subplot('position',[0.15, 0.13, 0.24, 0.24*7/8]);
    h = plot ( ...
              CCx, CCy, ...                 
                Cx,Cy, 'b', ...                 
                  [0 0],[-1 1], 'b', ...
                    [-1 1],[0 0], 'b', ...
                      EcircX, EcircY, 'm-', ...
                        Ux,Uy,'bo', ...
                          Ux(1), Uy(1), 'co', ...
                            Ue(1), Ue(2), 'r.' );
    beamcolor = [0.95 0.90 1.0];
    set (h(1),'color',beamcolor, 'linewidth',6);
    set (h(2:4),'color',[1 1 1]*0.7);
    set (h(5),'color',[1, 0.7, 0.7], 'linewidth',1.5);
    set (h(7),'markerfacecolor','c', 'markeredgecolor','b');
    for m=1:length(Ux),
        text (Ux(m)+0.1,Uy(m)+0.1,num2str(m),'fontsize',10);
    end
    set (gca,'layer','top');
    set (gca,'tickdir','out');
    set (gca,'ticklength',[0.03 0.025]);
    set (gca,'ytick',[-1 0 1]);
    set (gca,'xtick',[-1 0 1]);
    set (gca,'xlim',[-1.2 1.2]);
    set (gca,'ylim',[-1.2 1.2]);
    xlabel ('u_x');
    ylabel ('u_y');
    h = title ('Grating'); 
    set (h, 'fontweight','b');

%%%axis equal
%%%getfigure('plane-array (1)',[0.25 2.5 7 8], 550);

% Plot G(theta) zoomed around the max gain direction in the selected V-plane

subplot('position',[0.57,0.13,0.3350,0.22]);

    area ( thetadeg-Bdir*r2d, G_dB, dBmin, ...
            'facecolor',beamcolor,...
            'edgecolor', beamcolor); 
    hold on;
    plot (thetadeg-Bdir*r2d,G_dB,'k');
    hold off;
    set (gca,'layer','top');
    set (gca,'tickdir','out');
    set (gca,'ticklength',[0.025 0.025]);
    set (gca,'xlim',[theta_main1,theta_main2]*r2d);
    set (gca,'ylim',[dBmin,9]);
    h = title (sprintf('Gain at \\phi=%1.1f\\circ \\theta\\approx%1.1f\\circ',...
                    phi*r2d, Bdir*r2d)); 
    set (h, 'fontweight','b');
    xlabel ('\Delta\theta (\circ)');
    ylabel ('G / G_{ref} (dB)');

    s = sprintf('g=%+1.1fdB  w{\\approx}%1.2f\\circ',Gmax_dB,BW*r2d);
    x = theta_main1 + 0.03*(theta_main2-theta_main1);
    h = text (x*r2d,4,s); set(h,'fontsize',12);


if PLOT_3D
getfigure('plane-array (2)', [0.25 2.5 6 6], 400);
    az = Yphi;
    el = pi/2 - Yth;
    alpha = Ybw/2;
    planearray_plot3 ( az, el, alpha, phi, Ux, Uy, Uz );
end


if 0
getfigure('plane-array (3)', [0.25 2.5 6 6], 400);
    clf
    hold on
    plot_grating_zones ( Dx, Dy, deltaX, deltaY, 'c'); 
    h = plot ( ...
                Cx,Cy, 'b--', ...
                  [0 0],[-1 1], 'k', ...
                    [-1 1],[0 0], 'k', ...
                        Ux,Uy,'bo', ...
                          Ux(1), Uy(1), 'bo' ...
                                    );
    hold off;
    set (h(5),'markerfacecolor','blue');
    box on;    

%    for m=2:length(Ux),
%        text (Ux(m)+0.1,Uy(m)+0.1,num2str(m-1),'fontsize',10);
%    end
    set (gca,'layer','top');
    set (gca,'tickdir','out');
    set (gca,'ticklength',[0.02 0.025]);
    set (gca,'ytick',[-1 0 1]);
    set (gca,'xtick',[-1 0 1]);
    set (gca,'xlim',[-1.2 1.2]);
    set (gca,'ylim',[-1.2 1.2]);
    axis equal
    xlabel ('u_x');
    ylabel ('u_y');
%    h = title ('Grating zones'); 
%    set (h, 'fontweight','b');


end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               SUBFUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function plot_grating_zones ( Dx, Dy, deltaX, deltaY, color) 
%---------------------------------------------------------------
% Add crating zone boundaries on the unit circle.
%
%---------------------------------------------------------------
ux = rem ( deltaX/(2*pi*Dx), 2*pi);
if ux > pi, ux = ux - 2*pi; end  
n1 = ceil ( -Dx - ux*Dx - 0.5);
n2 = floor ( Dx - ux*Dx - 0.5);
for n = n1:n2
    x = ux + (2*n+1)/(2*Dx);
    y = sqrt(1 - x^2);
    plot ( [x,x],[-y y],color);
end
uy = rem ( deltaY/(2*pi*Dy), 2*pi);
if uy > pi, uy = uy - 2*pi; end  
n1 = ceil ( -Dy - uy*Dy - 0.5);
n2 = floor ( Dy - uy*Dy - 0.5);
for n = n1:n2
    y = uy + (2*n+1)/(2*Dy);
    x = sqrt(1 - y^2);
    plot ( [-x,x],[y y],color);
end
%end plot_grating_zones


function [ymax, k, k1, k2] = beamparam (y, k0, ref)
%---------------------------------------------------------------
% Maximum value in the neighbourhood of the index k0 (such that
% y(k0) is near the actual max), 
% and the indexes k1 and k2 of the nearest the points
% P1 and P2 both sides of the maximum  nearest to the max where
% y gets lower than the given ref level. The ref level is relative
% to ymax, so that ref = 1.0 means ymax.
%---------------------------------------------------------------

k = nearestmax (y, k0);
ymax = y(k);

m1 = min (find (y(k:-1:1) <= ref*ymax));
m2 = min (find (y(k:end) <= ref*ymax));
k1 = k - m1 + 1;
k2 = k + m2 - 1;

dk = abs(k2-k1);
k11 = max([1, k - dk]);
k22 = min([length(y), k + dk]); 

K = [1:k11 k22:length(y)];


function k = nearestmax(y,k0)
%----------------------------------------------------------------------
% k = nearestmax(y,k0)
%
% Find the (local) max of y that is nearest to the given index k0
% If y does not increase in either direction from k0, we return k0.
%
% 18-Oct-2005 Jm
%----------------------------------------------------------------------

N = length(y);
if k0 > N, k0 = N; end
if k0 < 1, k0 = 1; end

k = k0;
if N == 1
    k = k0;
    return
end

if k == 1
    for k=1:N-1
        if y(k+1) <= y(k), break, end
    end
    if y(k+1) > y(k), k=k+1; end
elseif k == N
    for k=N:-1:2
        if y(k-1) <= y(k), break, end
    end
    if y(k-1) > y(k), k = k-1; end
elseif y(k0+1) > y(k0)
    for k=k0:N-1
        if y(k+1) <= y(k), break, end
    end
    if y(k+1) > y(k), k=k+1; end
elseif y(k0-1) > y(k0)    
    for k=k0:-1:2
        if y(k-1) <= y(k), break, end
    end
    if y(k-1) > y(k), k = k-1; end
end    



